/// This is part of the MTP Implementation
/// (C) Daniel Kirstenpfad 2004-2005
/// 
/// 01.01.2005	-	initial Release																					-	Daniel Kirstenpfad
/// 02.01.2005	-	implemented datastructures from ptp specification						-	Daniel Kirstenpfad
using System;

namespace MTP
{
	#region Enumerations
	/// <summary>
	/// Modes allow the device to express different states with different capabilities. If the device supports only one mode, this field should contain the value 0x00000000.
	/// The current functional mode is also contained in a device property. In order to change the functional mode of the device, a session should be opened and the appropriate device property updated (if allowed). More information about device properties is available later in this document.
	/// </summary>
	public enum FunctionalModes
	{
		StandardMode = 0x0000,
		SleepState = 0x0001,
		NonResponsivePlayback = 0xC001,
		ResponsivePlayback = 0xC002
	}

	/// <summary>
	/// This identifies the PTP vendor-extension version in use by this device. For MTP devices implemented under this specification, this should contain the value 0x00000006 (the Microsoft Vendor Extension ID)
	/// </summary>
	public enum VendorIDs
	{
		Eastman_Kodak = 0x00000001,
		Seiko_Epson = 0x00000002,
		Agilent = 0x00000003,
		Polaroid = 0x00000004,
		Agfa_Gevaert = 0x00000005,
		Microsoft = 0x00000006,
		Equinox = 0x00000007,
		Viewquest = 0x00000008,
		ST_Microelectronics = 0x00000009,
		Nikon = 0x0000000A,
		Canon = 0x0000000B
	}

	/// <summary>
	/// Response Codes
	/// </summary>
	public enum ResponseCodes
	{
		Undefined = 0x2000,
		OK = 0x2001,
		GeneralError = 0x2002,
		SessionNotOpen = 0x2003,
		InvalidTransactionID = 0x2004,
		OperationNotSupported = 0x2005,
		ParameterNotSupported = 0x2006,
		IncompleteTransfer = 0x2007,
		InvalidStorageID = 0x2008,
		InvalidObjectHandle = 0x2009,
		DevicePropNotSupported = 0x200A,
		InvalidObjectFormatCode = 0x200B,
		StoreFull = 0x200C,
		ObjectWriteProtected = 0x200D,
		StoreReadOnly = 0x200E,
		AccessDenied = 0x200F,
		NoThumbnailPresent = 0x2010,
		SelfTestFailed = 0x2011,
		PartialDeletion = 0x2012,
		StoreNotAvailable = 0x2013,
		SpecificationByFormatUnsupported = 0x2014,
		NoValidObjectInfo = 0x2015,
		InvalidCodeFormat = 0x2016,
		UnknownVendorCode = 0x2017,
		CaptureAlreadyTerminated = 0x2018,
		DeviceBusy = 0x2019,
		InvalidParentObject = 0x201A,
		InvalidDevicePropFormat = 0x201B,
		InvalidDevicePropValue = 0x201C,
		InvalidParameter = 0x201D,
		SessionAlreadyOpened = 0x201E,
		TransactionCanceled = 0x201F,
		SpecificationOfDestinationUnsupported = 0x2020
	}

	/// <summary>
	/// This field identifies by datacode all operations that this device supports in the current functional mode.
	/// </summary>
	public enum OperationCodes
	{
		Undefined  = 0x1000,
		GetDeviceInfo = 0x1001,
		OpenSession = 0x1002,
		CloseSession = 0x1003,
		GetStorageIDs = 0x1004,
		GetStorageInfo = 0x1005,
		GetNumObjects = 0x1006,
		GetObjectHandles = 0x1007,
		GetObjectInfo = 0x1008,
		GetObject = 0x1009,
		GetThumb = 0x100A,
		DeleteObject = 0x100B,
		SendObjectInfo = 0x100C,
		InitiateCapture = 0x100E,
		FormatStore = 0x100F,
		ResetDevice = 0x1010,
		SelfTest = 0x1011,
		SetObjectProtection = 0x1012,
		PowerDown = 0x1013,
		GetDevicePropDesc = 0x1014,
		GetDevicePropValue = 0x1015,
		SetDevicePropValue = 0x1016,
		ResetDevicePropValue = 0x1017,
		TerminateOpenCapture = 0x1018,
		MoveObject = 0x1019,
		CopyObject = 0x101A,
		GetPartialObject = 0x101B,
		InitiateOpenCapture = 0x101C
	}

	/// <summary>
	/// As MTP is file-system-agnostic, the usual method of overloading file extensions to determine file type is not a reliable method of identifying device contents. Instead, objects formats are identified using predefined ObjectFormat datacodes.
	/// </summary>
	public enum ObjectFormats
	{
		Undefined = 0x3000,
		Association = 0x3001,
		Script = 0x3002,
		Executable = 0x3003,
		Text = 0x3004,
		HTML = 0x3005,
		DPOF = 0x3006,
		AIFF = 0x3007,
		WAV = 0x3008,
		MP3 = 0x3009,
		AVI = 0x300A,
		MPEG = 0x300B,
		ASF = 0x300C,
		defined = 0x3800,
		EXIF_JPEG = 0x3801,
		TIFF_EP = 0x3802,
		FlashPix = 0x3803,
		BMP = 0x3804,
		CIFF = 0x3805,
		Undefined_reserved = 0x3806,
		GIF = 0x3807,
		JFIF = 0x3808,
		CD = 0x3809,
		PICT = 0x380A,
		PNG = 0x380B,
		Undefined2_reserved = 0x380C,
		TIFF = 0x380D,
		TIFF_IT = 0x380E,
		JP2 = 0x380F,
		JPX = 0x3810
	}

	/// <summary>
	/// Eventcodes
	/// </summary>
	public enum EventCodes
	{
		Undefined = 0x4000,
		CancelTransaction = 0x4001,
		ObjectAdded = 0x4002,
		ObjectRemoved = 0x4003,
		StoreAdded = 0x4004,
		StoreRemoved = 0x4005,
		DevicePropChanged = 0x4006,
		ObjectInfoChanged = 0x4007,
		DeviceInfoChanged = 0x4008,
		RequestObjectTransfer = 0x4009,
		StoreFull = 0x400A,
		DeviceReset = 0x400B,
		StorageInfoChanged = 0x400C,
		CaptureComplete = 0x400D,
		UnreportedStatus = 0x400E
	}

	/// <summary>
	/// Associations may be used to represent various collections of objects. The implied meaning is conveyed by the Association Type field in the ObjectInfo dataset, and is also exposed in the Association Type object property (for more information, see section 12 "Object Properties"). In MTP, much of this functionality is superseded by Object References.
	/// </summary>
	public enum AssociationCodes
	{
		Undefined = 0x0000,
		Generic_Folder = 0x0001, // Association objects with this Association Type represent hierarchical folders rooted on a particular storage. This provides a mechanism of exposing a file hierarchy on the device without relying on any path or naming conventions specific to a particular filesystem. The AssociationDesc field of a Generic Folder Association may contain either 0x00000000 or 0x00000001. If it contains a value of 0x00000001, this indicates that it is a bi-directionally linked folder, and must have Object References to each object "contained" by this association (each object which contains this Associations ObjectHandle in the ParentID field of its ObjectInfo dataset).
		Album = 0x0002,  // This Accociation Type is PTP-specific. For more information, please refer to PIMA 15740, section 6.4.1.
		TimeSequence = 0x0003, // This Accociation Type is PTP-specific. For more information, please refer to PIMA 15740, section 6.4.1.
		Horizontal_Panoramic = 0x0004, // This Accociation Type is PTP-specific. For more information, please refer to PIMA 15740, section 6.4.1.
		Vertical_Panoramic = 0x0005, // This Accociation Type is PTP-specific. For more information, please refer to PIMA 15740, section 6.4.1.
		Panoramic_2D = 0x0006,  // This Accociation Type is PTP-specific. For more information, please refer to PIMA 15740, section 6.4.1.
		Ancillary_Data = 0x0007 // This Accociation Type is PTP-specific. For more information, please refer to PIMA 15740, section 6.4.1.
	}

	/// <summary>
	/// This field identifies the physical nature of the storage described by this dataset. If the StorageType is read-only memory (0x0001 or 0x0002), it supersedes any protection status indicated by the Access Capability field.
	/// </summary>
	public enum StorageTypes
	{
		Undefined = 0x0000,
		Fixed_ROM = 0x0001,
		Removable_ROM = 0x0002,
		Fixed_RAM = 0x0003,
		Removable_RAM = 0x0004
	}

	/// <summary>
	/// This field identifies the logical file system in use on this storage. This field may be used to define the file-naming conventions or directory structure conventions in use on this storage, as well as indicate support for a hierarchical file system.
	/// </summary>
	public enum FilesystemTypes
	{
		Undefined = 0x0000,
		Generic_flat = 0x0001,
		Generic_hierarchical = 0x0002,
		DCF = 0x0003
	}

	/// <summary>
	/// This field identifies any write-protection that globally affects this storage (this supersedes any protection status on individual objects). If the StorageType field indicates that this storage is defined as ROM (0x0001 or 0x0002), this field must contain the value 0x0002 (read-only without object deletion).
	/// </summary>
	public enum AccessCapabilities
	{
		Read_Write = 0x0000,
		Read_only_with_object_deletion = 0x0001,
		Read_only_without_object_deletion = 0x0002
	}

	/// <summary>
	///  An optional field that identifies the write-protection status of a data object. This field cannot be updated directly; instead, the SetProtection operation should be used. If the device does not support object protection, this field should always contain 0x0000, and the SetProtection operation should not be supported.
	/// </summary>
	public enum ProtectionStatus
	{
		No_Protection = 0x0000, // This object has no protection, it may be modified/deleted arbitrarily, and its properties may be modified freely.
		Read_Only = 0x0001, // This object cannot be deleted or modified, and all properties of this object cannot be modified by the initiator. (However, properties may be modified by the device that contains the object.)
		Read_Only_Data = 0x8002, // This objects binary component cannot be deleted or modified, however, any object properties may be modified if allowed by the object property constraints.
		Non_Transferrable_Data = 0x8003 // This objects properties may be read and modified, and it may be moved or deleted on the device, but this objects binary data may not be retrived from the device using a GetObject operation.
	}
	#endregion

	#region Structures
	/// <summary>
	/// The DeviceInfo dataset is used to provide a description of the device. This dataset can be obtained using the GetDeviceInfo operation without first initiating a session, and is mostly static. Should any value in this dataset be changed within a session, a DeviceInfoChanged event shall be issued to each connected device, and each device should re-acquire the DeviceInfo dataset to determine the updated values..
	/// </summary>
	public struct DeviceInfo
	{
		public Int16 StandardVersion; // This identifies the PTP version this device can support in hundredths. For MTP devices implemented under this specification, this should contain the value 100 (representing 1.00).
		public VendorIDs VendorExtensionID; // This identifies the PTP vendor-extension version in use by this device. For MTP devices implemented under this specification, this should contain the value 0x00000006 (the Microsoft Vendor Extension ID).
		public Int16 VendorExtensionVersion; // This identifies the version of the MTP standard this device supports. It is expressed in hundredths. The final version of this specification will identify the correct value to place in this field.
		public string VendorExtensionDesc; // This string is used to identify any extension sets applied to MTP, and is discussed in length later in this specification.
		public FunctionalModes FunctionalMode; //Modes allow the device to express different states with different capabilities. If the device supports only one mode, this field should contain the value 0x00000000.
		public OperationCodes[] OperationsSupported; // This field identifies by datacode all operations that this device supports in the current functional mode.
		public EventCodes[] EventsSupported; // This field identifies by datacode all events that this device can generate in the current functional mode.
		public Array DevicePropertiesSupported; // This field identifies by datacode all device properties that this device supports in the current functional mode.
		public ObjectFormats[] CaptureFormats; // This field identifies by datacode the object format codes for each format that this device can generate independently (that is, without the content being placed on the device).
		public ObjectFormats[] PlaybackFormats; // This field identifies by datacode the object format codes for each format that this device can understand and parse if placed on the device. If the device can carry unidentified binary objects without understanding them, it should be sure to indicate this in this field by including the Undefined Object (0x3000) code.
		public string Manufacturer; // This optional string is a human-readable string identifying the manufacturer of this device.
		public string Model; // This optional string is a human-readable string identifying the model of this device.
		public string DeviceVersion; // This optional string identifies the software or firmware version of this device in a vendor-specific format.
		public string SerialNumber; // This string is required, and contains the device's serial number. Serial numbers are required to be unique among all devices sharing identical Model and Device Version fields (this field was optional in the PTP specification, but is required in MTP).	It is strongly suggested that the serial number always be represented by a 32 character hexadecimal string for legacy compatibility reasons.  This string should be exactly 32 characters, including any leading 0s, and does not require any prefix to identify it as hexadecimal (such as 0x).
	}

	public struct ObjectInfo
	{
		public Int32 StorageID; // The device storage on which the object defined by this dataset is located.
		public ObjectFormats ObjectFormat; // Every object format type is identified by an ObjectFormatCode.
		public Int16 ProtectionStatus; // An optional field that identifies the write-protection status of a data object. This field cannot be updated directly; instead, the SetProtection operation should be used. If the device does not support object protection, this field should always contain 0x0000, and the SetProtection operation should not be supported.
		public Int32 ObjectCompressedSize; // The size of the data component of the object in bytes.
		public ObjectFormats ThumbFormat; // These fields provide information about thumbnails of images on the device. In MTP, thumbnail functionality has been supplanted with the Representative Sample Object Properties, and in most cases these values should be duplicated there.
		public Int32 ThumbCompressedSize; // These fields provide information about thumbnails of images on the device. In MTP, thumbnail functionality has been supplanted with the Representative Sample Object Properties, and in most cases these values should be duplicated there.
		public Int32 ThumbPixWidth; // These fields provide information about thumbnails of images on the device. In MTP, thumbnail functionality has been supplanted with the Representative Sample Object Properties, and in most cases these values should be duplicated there.
		public Int32 ThumbPixHeight; // These fields provide information about thumbnails of images on the device. In MTP, thumbnail functionality has been supplanted with the Representative Sample Object Properties, and in most cases these values should be duplicated there.
		public Int32 ImagePixWidth; // If the defined object is an image, this field identifies the width in pixels of that image. This information is also available in MTP through the object property mechanism, but should be implemented here to maintain PTP-compatibility if desired.
		public Int32 ImagePixHeight; // If the defined object is an image, this field identifies the height in pixels of that image. This information is also available in MTP through the object property mechanism, but should be implemented here to maintain PTP-compatibility if desired.
		public Int32 ImageBitDepth; // If the defined object is an image, this field identifies the bit depth of that image. This information is also available in MTP through the object property mechanism, but should be implemented here to maintain PTP-compatibility if desired.
		public object ParentObject;	// If this object exists in a hierarchy, this field contains the Object Handle of the parent of this object. Only objects of type Association may be identified in this field. If this object is in the root or the device does not support Associations, this field should be set to 0x00000000.
		public AssociationCodes AssociationType; // This field is only used for objects of type Association, and indicates what type of Association it is. In MTP, the Generic Folder Association type is most commonly used, which has an Association Type code of 0x0001.
// TODO: AssociationDesc Typ nicht exakt definiert in der Protokoll Spezifikation
		public Array AssociationDesc; // This field is used only for objects of type Association, and provides additional information about the implementation of this particular type
		public Int32 SequenceNumber; // This field is used in PTP to further define certain non-folder types of associations. It is not generally used in MTP, but should be implemented by any device that desires the provided functionality and that wishes to interact with PTP Initiators.
		public string Filename; // A string containing the file name of this object, without including any directory or file system information. This string is also accessible and defined via an Object Property, and restrictions on its format may be identified in the Object Property Description for this object property.
		public string Date_Created;	// This field identifies the creation date of this object, and uses the DateTime string 
		public string Date_Modified; // This field identifies the date when this object was last modified, and uses the DateTime string 
		public string Keywords; // This field contains keywords associated with the object. Keywords should be delimited by a single space: " ". Keywords containing a space should replace the space with an underscore character: "_".
	}

	public struct StorageInfo
	{
		public StorageTypes StorageType; // This field identifies the physical nature of the storage described by this dataset. If the StorageType is read-only memory (0x0001 or 0x0002), it supersedes any protection status indicated by the Access Capability field.
		public FilesystemTypes FilesystemType; // This field identifies the logical file system in use on this storage. This field may be used to define the file-naming conventions or directory structure conventions in use on this storage, as well as indicate support for a hierarchical file system.
		public AccessCapabilities AccessCapability; // This field identifies any write-protection that globally affects this storage (this supersedes any protection status on individual objects). If the StorageType field indicates that this storage is defined as ROM (0x0001 or 0x0002), this field must contain the value 0x0002 (read-only without object deletion).
		public Int64 MaxCapacity; // This field identifies the maximum capacity in bytes of this storage. If this storage can be written to, that is, the storage type is not read-only and the access capability is read-write, then this field must contain an accurate value. If the storage is read-only, this field is optional.
		public Int64 FreeSpaceInBytes; // This field indicates how much space remains to be written to on the drive. If this storage can be written to, that is, the storage type is not read-only and the access capability is read-write, then this field must contain an accurate value. If the storage type is read-only, this field is optional. If FreeSpaceInBytes does not apply to this device or this storage, and it can be written to, this field may contain a value of 0xFFFFFFFF, and the FreeSpaceInObjects field may be used instead.
		public Int32 FreeSpaceInObjects; // This field indicates how many additional objects may be written to this device. This field should only be used if there is a reasonable expectation that the number of objects that remain to be written can be accurately predicted (for instance, if the device contains only one object type of fixed size.). If this field is not used, it should be set to 0xFFFFFFFF.
		public string StorageDescription; // This optional field contains a human-readable string identifying this storage, such as "256Mb SD Card" or "20Gb HDD". If unused, it should contain an empty string.
		public string Volume_Identifier; // This field contains a unique, programmatically relevant volume identifier, such as a serial number. This field may be up to 255 characters long, however, only the first 128 characters will be used to identify the device, and these first 128 characters must be unique for all storages. If this field does not contain a string in which the first 128 characters are unique, it must contain an empty string.
	}
	#endregion
}
